==> mutual_credit-5.0.x-dev.tar.gz: mutual_credit/modules/credit_commons/src/Form/SettingsForm.php <==
<?php

namespace Drupal\mcapi_credcom\Form;

use Drupal\mcapi_credcom\TrunkwardNodeRequester;
use Drupal\Core\Entity\Entity\EntityFormDisplay;
use Drupal\Core\Entity\Entity\EntityFormMode;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use CreditCommons\Exceptions\CCFailure;
use Drupal\Component\Uuid\Uuid;


class SettingsForm extends ConfigFormBase {

  private $entityTypeManager;
  private $traded;
  private $trunkwardNode;
  private $allWorkflows = [];

  /**
   * {@inheritDoc}
   */
  public function getFormID() {
    return 'mcapi_credcom_settings_form';
  }

  /**
   * Constructor
   *
   * @param $config_factory
   * @param EntityTypeManager $entity_type_manager
   * @param TrunkwardNodeRequester $trunkward_node
   */
  public function __construct($config_factory, EntityTypeManager $entity_type_manager, TrunkwardNodeRequester $trunkward_node) {
    parent::__construct($config_factory);
    $this->entityTypeManager = $entity_type_manager;
    $this->config = $config_factory->get('mcapi_credcom.settings');
    $this->traded = (bool)$entity_type_manager->getStorage('mcapi_transaction')
      ->getQuery()
      ->accessCheck(FALSE)
      ->condition('type', 'remote')
      ->execute();
    $this->trunkwardNode = $trunkward_node;

    if ($trunkward_node->configured()) {
      $this->allWorkflows = $this->trunkwardNode->getWorkflows();
    }
  }

  static public function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('mcapi_credcom.trunkward_node')
    );
  }

  /**
   * {@inheritDoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->configFactory->get('mcapi_credcom.settings');

    $form['currency_id'] = [
      '#title' => $this->t('Currency used for intertrading'),
      '#type' => 'mcapi_currency_select',
      '#multiple' => FALSE,
      '#default_value' => $config->get('currency_id'),
      '#weight' => 0,
      '#disabled'=> $this->traded,
      '#required' => TRUE
    ];
    $form['local_curr_symbol'] = [
      '#title' => $this->t('Symbol for the local currency'),
      '#description' => $this->t('An html string which is placed before the numeric value.'),
      '#type' => 'textfield',
      '#default_value' => $config->get('local_curr_symbol'),
      '#weight' => 1,
      '#required' => TRUE
    ];
    $form['rate'] = [
      '#title' => $this->t('Conversion rate'),
      '#description' => $this->t('The number of (raw) local units to one unit on the parent ledger. Note that local units are stored as integers of the smallest subdivision, e.g. cents not dollars, minutes, not hours.'),
      '#type' => 'number',
      '#min' => 0,
      '#step' => 0.01,
      '#default_value' => $config->get('rate'),
      '#disabled'=> $this->traded,
      '#weight' => 2,
    ];
    $form['wallet_name'] = [
      '#title' => $this->t('Friendly name of the intertrading wallet'),
      '#type' => 'textfield',
      '#default_value' => $config->get('wallet_name'),
      '#weight' => 3,
      '#required' => TRUE
    ];
    $url = $config->get('trunkwards_node_url');
    if ($url and $ccuser = $config->get('cc_user')) {
      $connected = handshake($url, $ccuser, $ccauth);
    }

    $form['connection'] = [
      '#title' => $this->t('Connection to trunkwards node'),
      '#type' => 'details',
      '#open' => !isset($connected),
      '#weight' => 4,
      'trunkward_node_id' => [
        '#title' => $this->t('Trunkwards node ID'),
        '#description' => $this->t('The cc_user name the trunkward node uses to connect'),
        '#type' => 'textfield',
        '#default_value' => $url ?? $this->config->get('trunkward_node_id'),
        '#pattern' => '[a-z1-9]+',
        '#required' => TRUE,
        '#disabled'=> isset($connected),
        '#weight' => 0
      ],
      'trunkward_node_url' => [
        '#title' => $this->t('Trunkward node URL'),
        '#description' => $this->t('Must be https to protect the login credentials'),
        '#type' => 'textfield',
        '#default_value' => $this->config->get('trunkward_node_url'),
        '#placeholder' => 'https://',
        '#pattern' => 'https?:\/\/[a-z/.\-0-9]+', // https enforced except during development
        '#disabled'=> isset($connected),
        '#required' => TRUE,
        '#weight' => 1
      ],
      'cc_user' => [
        '#title' => $this->t('The name of the current site on the network'),
        '#description' => $config->get('cc_user') ? t('This cannot be changed.') : t('The account must exist already on the trunkwards node.'),
        '#type' => 'textfield',
        '#default_value' => $config->get('cc_user'),
        '#placeholder' => $this->t('lowercase with _ and no spaces.'),
        '#pattern' => '[a-z1-9_]+',
        '#weight' => 2,
        '#disabled'=> isset($connected),
        '#required' => TRUE
      ]
    ];
    $options = [];
    foreach ($this->allWorkflows as $wf) {
      if ($wf->creation->by) { // only 1stparty workflows.
        $options[$wf->id] = $wf->label .': '. $wf->summary;
      }
    }
    $form['workflows'] = [
      '#title' => $this->t('Supported Credit Commons workflows'),
      '#description' => $this->t("Users with %permission permission will be able to do these transactions."),
      '#type' => 'checkboxes',
      '#options' => $options,
      '#default_value' => $config->get('workflows'),
      '#weight' => 5
    ];
    return parent::buildForm($form, $form_state);
  }

  public function validateForm(array &$form, FormStateInterface $form_state) {
    $url = $form_state->getValue('trunkward_node_url');
    $ccuser = $form_state->getValue('cc_user');
    $ccauth = $form_state->getValue('cc_auth');
    if (!$this->handshake($url, $ccuser, $ccauth)) {
      $form_state->setError(
        $form['connection']['trunkward_node_url'],
        $this->t(
          'This node is not connected to %url. Check that the account exists and the name as password correspond.',
          ['%url' => $url]
        )
      );
    }
  }

  /**
   * {@inheritDoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->configFactory->getEditable('mcapi_credcom.settings')
      ->set('trunkward_node_id', $form_state->getValue('trunkward_node_id'))
      ->set('trunkward_node_url', $form_state->getValue('trunkward_node_url'))
      ->set('local_curr_symbol', $form_state->getValue('local_curr_symbol'))
      ->set('wallet_name', $form_state->getValue('wallet_name'))
      ->set('rate', $form_state->getValue('rate'))
      ->set('cc_user', $form_state->getValue('cc_user'))
      ->set('currency_id', $form_state->getValue('currency_id'))
      ->set('workflows', array_values($form_state->getValue('workflows')))
      ->save();
    $this->rebuildFirstpartyForms(array_filter($form_state->getValue('workflows')));
  }

  /**
   * {@inheritDoc}
   */
  protected function getEditableConfigNames() {
    return ['mcapi_credcom.settings'];
  }

  /*
   * Handshake the trunkward node using the given credentials (rather than the
   * saved credentials.)
   */
  private function handshake($url, $ccuser, $ccauth) {
    $this->trunkwardNode->nodeName = $ccuser;
    try {
      $this->trunkwardNode->handshake();
    } catch (CCFailure $ex) {
      return FALSE;
    }
    return TRUE;
  }

  /**
   * Make a transaction form for each selected remote workflow
   */
  function rebuildFirstpartyForms(array $active_workflow_names) {
    $form_display_ids = $this->entityTypeManager->getStorage('entity_form_display')
      ->getQuery()
      ->condition('id', 'mcapi_transaction.mcapi_transaction.remote_', 'STARTS_WITH')
      ->execute();
    // delete unused form displays.
    foreach ($form_display_ids as $form_display_id) {
      $wf_name = substr($form_display_id, strlen('mcapi_transaction.mcapi_transaction.remote_'));
      if (empty($active_workflow_names[$wf_name])) {
        if ($form_display = EntityFormDisplay::load($form_display_id)) {
          $this->messenger()->addStatus($this->t('Deleted FormDisplay @name', ['@name' => $form_display->id()]));
          $form_display->delete();
        }
      }
      unset($active_workflow_names[$wf_name]);
    }
    // Create new form displays
    foreach ($active_workflow_names as $wf_name) {
      if (EntityFormDisplay::load('mcapi_transaction.mcapi_transaction.remote_'.$wf_name)) {
        return;
      }
      // Make a new EntityFormDisplay.
      // The EntityFormMode already exists, now make the EntityFormDisplay.
      $wf = $this->allWorkflows[$wf_name];
      $id = 'remote_'.$wf->id;
      $form_display_values = EntityFormDisplay::load('mcapi_transaction.mcapi_transaction.default')->toArray();
      $form_display_values['mode'] = $id;
      $form_display_values['id'] = 'mcapi_transaction.mcapi_transaction.'.$id;
      unset($form_display_values['uuid']);
      // Create a new entity with the original values.
      $form_display = EntityFormDisplay::create($form_display_values);
      $form_display->setComponent(in_array('payer', $wf->creation->by) ? 'payee' : 'payer',
        [
          'region' => 'content',
          'type' => 'cc_wallet',
          'settings' => ['size' => 30, 'placeholder' => 'path/to/account'],
          'weight' => 3
        ]
      )->setComponent('worth',
        [
          'region' => 'content',
          'type' => 'worth',
          'settings' => ['exclude' => FALSE, 'currencies' => [$this->config->get('currency_id')]],
          'weight' => 3
        ]
      )->save();
      $url = Url::fromRoute('entity.entity_form_display.mcapi_transaction.form_mode', ['form_mode_name' => $id]);
      $this->messenger()->addStatus($this->t(
        'Now go and <a href=":url">edit</a> the new remote "@label" form.',
        [':url' => $url->toString(), '@label' => $wf->id]
      ));
    }

  }


}

==> mutual_credit-5.2.x-dev.tar.gz: mutual_credit/modules/credit_commons/src/Form/SettingsForm.php <==
<?php

namespace Drupal\mcapi_credcom\Form;

use Drupal\mcapi_credcom\TrunkwardNodeRequester;
use Drupal\Core\Entity\Entity\EntityFormDisplay;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use GuzzleHttp\Client;

/**
 * @deprecated
 */
class SettingsForm extends ConfigFormBase {

  private $entityTypeManager;
  private $traded;
  private $allWorkflows = [];
  private \Drupal\Core\Config\Config $config;

  /**
   * {@inheritDoc}
   */
  public function getFormID() {
    return 'mcapi_credcom_settings_form';
  }

  /**
   * Constructor
   *
   * @param $config_factory
   * @param EntityTypeManager $entity_type_manager
   * @param TrunkwardNodeRequester $trunkward_node
   */
  public function __construct($config_factory, EntityTypeManager $entity_type_manager) {
    parent::__construct($config_factory);
    $this->entityTypeManager = $entity_type_manager;
    $this->config = $config_factory->get('mcapi_credcom.settings');
    $this->traded = (bool)$entity_type_manager->getStorage('mcapi_transaction')
      ->getQuery()
      ->accessCheck(FALSE)
      ->condition('type', 'remote')
      ->execute();
  }

  static public function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritDoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['trunkward_node_url'] = [
      '#title' => t('Trunkward node URL'),
      '#description' => t('Must be https to protect the login credentials.'),
      '#type' => 'textfield',
      '#default_value' => $this->config->get('trunkward_node_url') ?? '',
      '#placeholder' => 'https://',
      '#pattern' => 'https?:\/\/[a-z/.\-0-9]+', // https enforced except during development
      '#required' => TRUE,
      '#disabled' => (bool)$this->config->get('trunkward_node_url'),
      '#weight' => 2
    ];

    $path = $this->config->get('trunkward_node_path');
    if (!$path and $url = $this->config->get('trunkward_node_url')) {
      $path = $this->checkNode($url, $form_state);
    }

    $form['trunkward_node_path'] = [
      '#title' => $this->t('Absolute path of trunkward node'),
      '#description' => $this->t('populated automatically when the trunkward account is created.'),
      '#type' => 'textfield',
      '#default_value' => $path,
      '#weight' => 3,
      '#disabled' => TRUE
    ];
    return parent::buildForm($form, $form_state);
  }

  public function validateForm(array &$form, FormStateInterface $form_state) {
    $form_state->setValue(
      'trunkward_node_path',
      $this->checkNode($form_state->getValue('trunkward_node_url'), $form_state)
    );
  }

  public function checkNode($url, FormStateInterface $form_state) : string {
    $client = new Client(['base_uri' => $url]);
    try {
      $result = $client->get('/about')->getBody();
    }
    catch(\Throwable $e) {
      $form_state->setErrorByName('trunkward_node_path', $e->getMessage());
      return '';
    }
    $payload = json_decode(strval($result));
    return implode('/', $payload->data->absolute_path);
  }

  /**
   * {@inheritDoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
    $trunkward_node_name = $form_state->getValue('trunkward_node_path');
    $pos = strrpos($trunkward_node_name,  '/');
    if(is_int($pos)) {
      $trunkward_node_name = substr($trunkward_node_name, $pos +1);
    }
    $this->configFactory->getEditable('mcapi_credcom.settings')
      ->set('trunkward_node_path', $form_state->getValue('trunkward_node_path'))
      ->set('trunkward_node_url', $form_state->getValue('trunkward_node_url'))
      ->set('trunkward_node_name', $trunkward_node_name)
      ->save();
  }

  /**
   * {@inheritDoc}
   */
  protected function getEditableConfigNames() {
    return ['mcapi_credcom.settings'];
  }

  /**
   * Make a transaction form for each selected remote workflow
   */
  function rebuildFirstpartyForms(array $active_workflow_names) {
    $form_display_ids = $this->entityTypeManager->getStorage('entity_form_display')
      ->getQuery()
      ->condition('id', 'mcapi_transaction.mcapi_transaction.remote_', 'STARTS_WITH')
      ->execute();
    // delete unused form displays.
    foreach ($form_display_ids as $form_display_id) {
      $wf_name = substr($form_display_id, strlen('mcapi_transaction.mcapi_transaction.remote_'));
      if (empty($active_workflow_names[$wf_name])) {
        if ($form_display = EntityFormDisplay::load($form_display_id)) {
          $this->messenger()->addStatus($this->t('Deleted FormDisplay @name', ['@name' => $form_display->id()]));
          $form_display->delete();
        }
      }
      unset($active_workflow_names[$wf_name]);
    }
    // Create new form displays
    foreach ($active_workflow_names as $wf_name) {
      if (EntityFormDisplay::load('mcapi_transaction.mcapi_transaction.remote_'.$wf_name)) {
        return;
      }
      // Make a new EntityFormDisplay.
      // The EntityFormMode already exists, now make the EntityFormDisplay.
      $wf = $this->allWorkflows[$wf_name];
      $id = 'remote_'.$wf->id;
      $form_display_values = EntityFormDisplay::load('mcapi_transaction.mcapi_transaction.default')->toArray();
      $form_display_values['mode'] = $id;
      $form_display_values['id'] = 'mcapi_transaction.mcapi_transaction.'.$id;
      unset($form_display_values['uuid']);
      // Create a new entity with the original values.
      $form_display = EntityFormDisplay::create($form_display_values);
      $form_display->setComponent(in_array('payer', $wf->creation->by) ? 'payee' : 'payer',
        [
          'region' => 'content',
          'type' => 'credcom_wallet',
          'settings' => ['size' => 30, 'placeholder' => 'path/to/account'],
          'weight' => 3
        ]
      )->setComponent('worth',
        [
          'region' => 'content',
          'type' => 'worth',
          'settings' => ['exclude' => FALSE, 'currencies' => [$this->config->get('currency_id')]],
          'weight' => 3
        ]
      )->save();
      $url = Url::fromRoute('entity.entity_form_display.mcapi_transaction.form_mode', ['form_mode_name' => $id]);
      $this->messenger()->addStatus($this->t(
        'Now go and <a href=":url">edit</a> the new remote "@label" form.',
        [':url' => $url->toString(), '@label' => $wf->id]
      ));
    }

  }


}
